// ---------- Cash System Script ----------
// ---------- Created by VRocker ----------

g_hashCash <- HashTable( "Cash" );
g_hashBank <- HashTable( "Bank" );

function LoadMoneyHashes()
{
	g_hashCash.Load( "Hashes/CashSystem/Cash.hsh" );
	g_hashBank.Load( "Hashes/CashSystem/Bank.hsh" );
}

function SaveMoneyHashes()
{
	g_hashCash.Save( "Hashes/CashSystem/Cash.hsh" );
	g_hashBank.Save( "Hashes/CashSystem/Bank.hsh" );
}

function CloseMoneyHashes()
{
	g_hashCash.Close();
	g_hashBank.Close();
	
	g_hashCash <- null;
	g_hashBank <- null;
}

// Cash stuff
function GetPlayerCash( name )
{
	local cash = g_hashCash.Get( name.tolower() );
	if ( cash != null )
		return cash;
		
	return 0;
}

function SetPlayerCash( name, ammount )
{
	g_hashCash.Add( name.tolower(), ammount );
	
	// Updates the cash value on screen
	// We do a FindPlayer rather than passing the pointer so values can be set when the player is not ingame
	local plr = FindPlayer( name );
	if ( plr )
		plr.Cash = ammount;
}

// Bank Stuff
function GetPlayerBank( name )
{
	local bank = g_hashBank.Get( name );
	if ( bank != null )
		return bank;
		
	return 0;
}

function SetPlayerBank( name, ammount )
{
	g_hashBank.Add( name, ammount );
}

function IncPlayerBank( name, ammount )
{
	g_hashBank.Inc( name, ammount );
}

function DecPlayerBank( name, ammount )
{
	g_hashBank.Dec( name, ammount );
}

function ProcessCashSystemPlayerJoin( plr )
{
	if ( g_hashCash.Get( plr.Name.tolower() ) == null )
	{
		// The player is new to the server, give him $2000
		SetPlayerCash( plr.Name, 2000 );
		MessagePlayer( "To get you started, you have been given $2000", plr );
	}
	
	// Updates the onscreen cash value
	plr.Cash = GetPlayerCash( plr.Name );
}

function ProcessCashSystemCommands( plr, level, cmd, args )
{
	if ( cmd == "cash" )
	{
		if ( !args )
			Message( plr + " currently has $" + GetPlayerCash( plr.Name ) );
		else
		{
			// Checking somebody elses cash
			local plrText = GetTok( args, " ", 1 );
			local plr2 = null;
			
			if ( IsNum( plrText ) ) plr2 = FindPlayer( plrText.tointeger() );
			else plr2 = FindPlayer( plrText );
			
			if ( plr2 )
			{
				Message( plr2 + " currently has $" + GetPlayerCash( plr2.Name ) );
			}
			else MessagePlayer( "Invalid player", plr, COL_CMDERROR );
		}
		
		return true;
	}
	else if ( cmd == "bank" )
	{
		if ( !args )
			Message( plr + " currently has $" + GetPlayerBank( plr.Name ) + " in the bank" );
		else
		{
			// Checking somebody elses cash
			local plrText = GetTok( args, " ", 1 );
			local plr2 = null;
			
			if ( IsNum( plrText ) ) plr2 = FindPlayer( plrText.tointeger() );
			else plr2 = FindPlayer( plrText );
			
			if ( plr2 )
			{
				Message( plr2 + " currently has $" + GetPlayerBank( plr2.Name ) + " in the bank" );
			}
			else MessagePlayer( "Invalid player", plr, COL_CMDERROR );
		}
		return true;
	}
	
	// Adding money to the bank
	else if ( ( cmd == "bank+" ) || ( cmd == "b+" ) || ( cmd == "deposit" ) )
	{
		// Check if the player is at the bank
		if ( GetAreaName( plr.Pos.x, plr.Pos.y ) == "Bank" )
		{
			if ( !args )
			{
				// Deposit all money to the bank
				local cash = GetPlayerCash( plr.Name );
				
				MessagePlayer( "Depositing $" + cash + " to your bank account", plr );
				
				// Set the players bank to the cash they are currently holding
				IncPlayerBank( plr.Name, cash );
				// Reset the players cash to 0 since it was all deposited
				SetPlayerCash( plr.Name, 0 );
				
				// To be safe, we save the hashes when money is deposited
				SaveMoneyHashes();
			}
			else
			{
				// Deposit a set ammount of cash
				if ( IsNum( args ) )
				{
					local ammount = args.tointeger();
					local cash = GetPlayerCash( plr.Name );
					
					if ( ammount <= cash )
					{
						MessagePlayer( "Depositing $" + ammount + " to your bank account", plr );
						
						// Set the players bank to the ammount specified
						IncPlayerBank( plr.Name, ammount );
						// Set the players cash to the new ammount
						SetPlayerCash( plr.Name, cash - ammount );
						
						// To be safe, we save the hashes when money is deposited
						SaveMoneyHashes();
					}
					else
						MessagePlayer( "You cannot deposit more cash than you have", plr, COL_CMDERROR );
				}
				else
					MessagePlayer( "Invalid arguments (/" + cmd + " <ammount>)", plr, COL_CMDERROR );
			}
		}
		else
			MessagePlayer( "You must be at the bank to deposit money", plr, COL_CMDERROR );
		
		return true;
	} // End 'deposit' command
	
	// Withdrawing money from the bank
	else if ( ( cmd == "bank-" ) || ( cmd == "b-" ) || ( cmd == "withdraw" ) )
	{
		// Check if the player is at the bank
		if ( GetAreaName( plr.Pos.x, plr.Pos.y ) == "Bank" )
		{
			if ( !args )
			{
				// Deposit all money to the bank
				local bank = GetPlayerBank( plr.Name );
				local cash = GetPlayerCash( plr.Name );
				
				MessagePlayer( "Withdrawing $" + bank + " from your bank account", plr );
				
				// Set the players cash to the ammount withdrawn
				SetPlayerCash( plr.Name, cash + bank );
				// Reset the players bank to 0 since it was all withdrawn
				SetPlayerBank( plr.Name, 0 );
				
				// To be safe, we save the hashes when money is withdrawn
				SaveMoneyHashes();
			}
			else
			{
				// Deposit a set ammount of cash
				if ( IsNum( args ) )
				{
					local ammount = args.tointeger();
					local bank = GetPlayerBank( plr.Name );
					local cash = GetPlayerCash( plr.Name );
					
					if ( ammount <= bank )
					{
						MessagePlayer( "Withdrawing $" + ammount + " from your bank account", plr );
										
						SetPlayerCash( plr.Name, cash + ammount );
						DecPlayerBank( plr.Name, ammount );
						
						// To be safe, we save the hashes when money is withdrawn
						SaveMoneyHashes();
					}
					else
						MessagePlayer( "You cannot withdraw more cash than you have in the bank", plr, COL_CMDERROR );
				}
				else
					MessagePlayer( "Invalid arguments (/" + cmd + " <ammount>)", plr, COL_CMDERROR );
			}
		}
		else
			MessagePlayer( "You must be at the bank to withdraw money", plr, COL_CMDERROR );
		
		return true;
	}
	
	else if ( cmd == "send" )
	{
		if ( args )
		{
			local numArgs = NumTok( args, " " );
			if ( numArgs == 2 )
			{
				local plrText = GetTok( args, " ", 1 );
				local ammountText = GetTok( args, " ", 2 );
				
				if ( IsNum( ammountText ) )
				{
					local ammount = ammountText.tointeger();
					local plr2 = null;
			
					if ( IsNum( plrText ) ) plr2 = FindPlayer( plrText.tointeger() );
					else plr2 = FindPlayer( plrText );
					
					if ( plr2 )
					{
						if ( plr2.ID != plr.ID )
						{
							local cash = GetPlayerCash( plr.Name );
							if ( ( cash - ammount ) > 2000 )
							{
								MessagePlayer( "You have sent $" + ammount + " to " + plr2, plr );
								MessagePlayer( plr + " has sent you $" + ammount, plr2 );
								
								local cash2 = GetPlayerCash( plr2.Name );
								SetPlayerCash( plr.Name, cash - ammount );
								SetPlayerCash( plr2.Name, cash2 + ammount );
								
								SaveMoneyHashes();
							}
							else
								MessagePlayer( "You cannot send money that leaves you with less than $2000", plr, COL_CMDERROR );
						}
						else
							MessagePlayer( "You cannot send money to yourself", plr, COL_CMDERROR );
					}
					else 
						MessagePlayer( "Invalid player", plr, COL_CMDERROR );
				}
				else
					MessagePlayer( "Invalid arguments (/send <player> <ammount>)", plr, COL_CMDERROR );
			}
			else
				MessagePlayer( "Invalid arguments (/send <player> <ammount>)", plr, COL_CMDERROR );
		}
		else
			MessagePlayer( "Invalid arguments (/send <player> <ammount>)", plr, COL_CMDERROR );
			
		return true;
	}
	
	else if ( cmd == "wire" )
	{
		// Check if the player is at the bank
		if ( GetAreaName( plr.Pos.x, plr.Pos.y ) == "Bank" )
		{
			if ( args )
			{
				local numArgs = NumTok( args, " " );
				if ( numArgs == 2 )
				{
					local plrText = GetTok( args, " ", 1 );
					local ammountText = GetTok( args, " ", 2 );
					
					if ( IsNum( ammountText ) )
					{
						local ammount = ammountText.tointeger();
						local plr2 = null;
			
						if ( IsNum( plrText ) ) plr2 = FindPlayer( plrText.tointeger() );
						else plr2 = FindPlayer( plrText );
					
						if ( plr2 )
						{
							if ( plr2.ID != plr.ID )
							{
								local bank = GetPlayerBank( plr.Name );
								if ( ( bank - ammount ) > 2000 )
								{
									MessagePlayer( "You have wired $" + ammount + " to " + plr2, plr );
									MessagePlayer( plr + " has wired $" + ammount + " to your bank account", plr2 );
									
									local bank2 = GetPlayerBank( plr2.Name );
									SetPlayerBank( plr.Name, bank - ammount );
									SetPlayerBank( plr2.Name, bank2 + ammount );
								
									SaveMoneyHashes();
								}
								else
									MessagePlayer( "You cannot wire money that leaves you with less than $2000", plr, COL_CMDERROR );
							}
							else
								MessagePlayer( "You cannot wire money to yourself", plr, COL_CMDERROR );
						}
						else 
							MessagePlayer( "Invalid player", plr, COL_CMDERROR );
					}
					else
						MessagePlayer( "Invalid arguments (/wire <player> <ammount>)", plr, COL_CMDERROR );
				}	
				else
					MessagePlayer( "Invalid arguments (/wire <player> <ammount>)", plr, COL_CMDERROR );
			}
			else
				MessagePlayer( "Invalid arguments (/wire <player> <ammount>)", plr, COL_CMDERROR );
		}
		else
			MessagePlayer( "You must be at the bank to wire money", plr, COL_CMDERROR );
			
		return true;
	}
	
	return false;
}
